#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

require "db.pl";
require "http.pl";
require "common.pl";
require "url.pl";
require "config.pl";
require "op_chan.pl";


# 0: chan
# 1: count
# r: array of hash (url_id, url_url, url_count, url_date, chan_id, chan_name,
#			nick_id, nick_name)
sub get_latest($$)
{
	my ($chan, $count) = @_;
	my ($query, $result);
	my @ret;

	$count = &Common::min($count, 100);
	my $date_fmt = 'YYYY-MM-DD"T"HH24:MI:SS.MS"Z"';
	my $to_char = "TO_CHAR(urls.time, '$date_fmt')";
	my $chan_q = defined($chan) ? " chans.id = $chan AND " : "";
	$query = "SELECT urls.id, urls.url, urls.count, $to_char AS time, " .
				"chans.id, chans.chan, " .
				"nicks.id, nicks.nick " .
			"FROM urls, chans, nicks " .
			"WHERE $chan_q urls.nick = nicks.id " .
				"AND urls.chan = chans.id " .
			"ORDER BY urls.id DESC " .
			"LIMIT $count";
	$result = &DB::query($query);
	($result->ntuples == 0) && return ();

	while (my ($url_id, $url_url, $url_count, $url_date,
				$chan_id, $chan_name,
				$nick_id, $nick_name) = $result->fetchrow) {
		my %t = (
			'url_id' => $url_id,
			'url_url' => $url_url,
			'url_date' => $url_date,
			'url_count' => $url_count,
			'chan_id' => $chan_id,
			'chan_name' => $chan_name,
			'nick_id' => $nick_id,
			'nick_name' => $nick_name
			);
		push(@ret, \%t);
	}
	return @ret;
}


# 0: type: 0 = feed, 1 = entry
# 1: channel (can be undef)
# 2: latest url_id
# r: atom::id
sub get_id($$$) {
	my ($type, $chan, $url_id) = @_;
	my $uri = &Conf::base_url();
	$uri =~ s,https?://,, if ($uri =~ /^http/);
	$uri =~ s:/+:,:g;
	my $tag = defined($chan) ? "chan-$chan" : "global";
	if ($type == 0) {
		return "tag:$uri:feed-$tag";
	} else {
		return "tab:$uri:entry-$tag-url-$url_id";
	}
}


sub print_entries($@) {
	my $chan = shift;
	foreach my $t (@_) {
		my %link = %{($t)};

		my $url = &URL::amp_url($link{'url_url'});
		my $title = &URL::amp_url(&HTML::truncate_url(
				$link{'url_url'}));
		my $id = &get_id(1, $chan, $link{'url_id'});
		print <<EOF;
<entry>
	<title>$title</title>
	<link href="$url" />
	<id>$id</id>
	<updated>$link{'url_date'}</updated>
	<summary>Pasted on $link{'chan_name'} by $link{'nick_name'}.
		Wanha count: $link{'url_count'}</summary>
</entry>
EOF
	}
}


sub main($)
{
	my $chan = &HTTP::get_numeric("chan");
	my @urls = &get_latest($chan, 50);

	my $title;
	if (defined($chan)) {
		my $chan = &Chan::get_name($chan);
		$title = "wanha links for channel $chan";
	} else {
		$title = "wanha links";
	}

	my @date = localtime();
	my $now = sprintf("%04d-%02d-%02dT%02d:%02d:%02dZ",
			$date[5] + 1900,	# YYYY
			$date[4],		# MM
			$date[3],		# DD
			$date[2],		# HH24
			$date[1],		# MI
			$date[0]);		# SS

	my $id = &get_id(0, $chan, $urls[0]{'url_id'});
	my $gen = &URL::amp_url(&Conf::base_url());
	my $baseurl = &Conf::base_url();

	print <<EOF;
Content-Type: application/atom+xml; charset=utf-8

<?xml version="1.0" encoding="UTF-8"?>
<feed xmlns="http://www.w3.org/2005/Atom">

<title>$title</title>
<link href="$baseurl" />
<updated>$now</updated>
<author>
	<name>Tommi Saviranta</name>
	<address>wnd\@iki.fi</address>
</author>
<id>$id</id>
<generator uri="$gen">wanha</generator>

EOF

	&print_entries($chan, @urls);

	print "</feed>\n";
}


&main();
